@@ -11,6 +11,7 @@
 #include <config.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <unistd.h>
 #include <string.h>
 #include <glib-object.h>
 
@@ -314,6 +315,19 @@ info_new_fn (void)
 	return (BonoboUIXmlData *) info;
 }
 
+static void
+widget_unref (GtkWidget **ref)
+{
+	GtkWidget *w;
+
+	g_return_if_fail (ref != NULL);
+	
+	if ((w = *ref)) {
+		*ref = NULL;
+		gtk_widget_unref (w);
+	}
+}
+
 static void
 info_free_fn (BonoboUIXmlData *data)
 {
@@ -324,7 +338,8 @@ info_free_fn (BonoboUIXmlData *data)
 		bonobo_object_release_unref (info->object, NULL);
 		info->object = CORBA_OBJECT_NIL;
 	}
-	info->widget = NULL;
+
+	widget_unref (&info->widget);
 
 	g_free (data);
 }
@@ -391,7 +406,6 @@ custom_widget_unparent (NodeInfo *info)
 		container = GTK_CONTAINER (info->widget->parent);
 		g_return_if_fail (container != NULL);
 
-		gtk_widget_ref (info->widget);
 		gtk_container_remove (container, info->widget);
 	}
 }
@@ -433,11 +447,13 @@ replace_override_fn (GObject        *object,
 	cmd_to_node_add_node    (engine, new, FALSE);
 
 /*	g_warning ("Replace override on '%s' '%s' widget '%p'",
-		   old->name, bonobo_ui_node_get_attr (old, "name"), old_info->widget);
+		   bonobo_ui_node_get_name (old),
+		   bonobo_ui_node_get_attr (old, "name"),
+		   old_info->widget);
 	info_dump_fn (old_info);
 	info_dump_fn (info);*/
 
-	/* Copy useful stuff across */
+	/* Copy useful stuff across & tranfer widget ref */
 	old_widget = old_info->widget;
 	old_info->widget = NULL;
 
@@ -502,7 +518,6 @@ prune_node (BonoboUIEngine *engine,
 #endif
 
 			gtk_widget_destroy (item);
-
 		} else {
 			if (save)
 				custom_widget_unparent (info);
@@ -511,7 +526,7 @@ prune_node (BonoboUIEngine *engine,
 		}
 
 		if (!save)
-			info->widget = NULL;
+			widget_unref (&info->widget);
 	}
 }
 
@@ -554,10 +569,15 @@ override_fn (GObject        *object,
 	fprintf (stderr, "Override '%s'\n", 
 		 bonobo_ui_xml_make_path (old));
 #endif
-	bonobo_ui_engine_prune_widget_info (engine, old, TRUE);
+	if (bonobo_ui_node_same_name (new, old)) {
+		replace_override_fn (object, new, old, engine);
 
-	cmd_to_node_remove_node (engine, old, FALSE);
-	cmd_to_node_add_node    (engine, new, FALSE);
+	} else {
+		bonobo_ui_engine_prune_widget_info (engine, old, TRUE);
+
+		cmd_to_node_remove_node (engine, old, FALSE);
+		cmd_to_node_add_node    (engine, new, FALSE);
+	}
 }
 
 static void
@@ -737,8 +757,15 @@ sub_component_destroy (BonoboUIEngine *engine, SubComponent *component)
 
 	if (component) {
 		g_free (component->name);
-		if (component->object != CORBA_OBJECT_NIL)
+		if (component->object != CORBA_OBJECT_NIL) {
+			CORBA_Environment ev;
+
+			CORBA_exception_init (&ev);
+			Bonobo_UIComponent_unsetContainer (component->object, &ev);
+			CORBA_exception_free (&ev);
+
 			bonobo_object_release_unref (component->object, NULL);
+		}
 		g_free (component);
 	}
 }
@@ -998,9 +1025,8 @@ bonobo_ui_engine_xml_get_prop (BonoboUIEngine *engine,
 			       const char     *prop,
 			       gboolean       *invalid_path)
 {
- 	char         *str;
+ 	const char   *str;
  	BonoboUINode *node;
-  	CORBA_char   *ret;
   
   	g_return_val_if_fail (BONOBO_IS_UI_ENGINE (engine), NULL);
 
@@ -1013,12 +1039,12 @@ bonobo_ui_engine_xml_get_prop (BonoboUIEngine *engine,
 		if (invalid_path)
 			*invalid_path = FALSE;
   
- 		str = bonobo_ui_node_get_attr (node, prop);
+ 		str = bonobo_ui_node_peek_attr (node, prop);
+
 		if (!str)
 			return NULL;
- 		ret = CORBA_string_dup (str);
- 		bonobo_ui_node_free_string (str);
- 		return ret;
+
+ 		return CORBA_string_dup (str);
   	}
 }
 
@@ -1033,7 +1059,7 @@ bonobo_ui_engine_xml_get_prop (BonoboUIEngine *engine,
  * node to an XML string, otherwise it dumps it and
  * its children.
  *
- * Return value: the XML string - use bonobo_ui_node_free_string to free
+ * Return value: the XML string - use CORBA_free to free
  **/
 CORBA_char *
 bonobo_ui_engine_xml_get (BonoboUIEngine *engine,
@@ -1052,7 +1078,7 @@ bonobo_ui_engine_xml_get (BonoboUIEngine *engine,
  	else {		
  		str = bonobo_ui_node_to_string (node, !node_only);
  		ret = CORBA_string_dup (str);
- 		bonobo_ui_node_free_string (str);
+ 		g_free (str);
  		return ret;
   	}
 }
@@ -1118,7 +1144,7 @@ bonobo_ui_engine_object_set (BonoboUIEngine   *engine,
 		bonobo_object_release_unref (info->object, ev);
 		if (info->widget)
 			gtk_widget_destroy (info->widget);
-		info->widget = NULL;
+		widget_unref (&info->widget);
 	}
 	
 	dprintf ("** Setting object %p on info %p\n", object, info);
@@ -1562,24 +1588,23 @@ set_cmd_attr (BonoboUIEngine *engine,
 
 #ifdef STATE_SYNC_DEBUG
 	fprintf (stderr, "Set '%s' : '%s' to '%s'",
-		 bonobo_ui_node_get_attr (cmd_node, "name"),
+		 bonobo_ui_node_peek_attr (cmd_node, "name"),
 		 prop, value);
 #endif
 
 	bonobo_ui_node_set_attr (cmd_node, prop, value);
 
 	if (event) {
-		GSList *updates;
-		char   *cmd_name = bonobo_ui_node_get_attr (
-			cmd_node, "name");
+		GSList     *updates;
+		const char *cmd_name;
 
+		cmd_name = bonobo_ui_node_peek_attr (cmd_node, "name");
 
 		updates = make_updates_for_command (
 			engine, NULL, cmd_node, cmd_name);
 
 		execute_state_updates (updates);
 
-		bonobo_ui_node_free_string (cmd_name);
 	} else {
 		BonoboUIXmlData *data =
 			bonobo_ui_xml_get_data (
@@ -1964,14 +1989,14 @@ hide_placeholder_if_empty_or_hidden (BonoboUIEngine *engine,
 	hide_placeholder_and_contents = txt && atoi (txt);
 
 	info = bonobo_ui_xml_get_data (engine->priv->tree, node);
-	has_visible_separator = info && info->widget
-		&& GTK_WIDGET_VISIBLE (info->widget);
+	has_visible_separator = info && info->widget &&
+		GTK_WIDGET_VISIBLE (info->widget);
 
 	if (hide_placeholder_and_contents)
 		hide_all_widgets (engine, node);
 
-	else if (has_visible_separator
-		 && !contains_visible_widget (engine, node))
+	else if (has_visible_separator &&
+		 !contains_visible_widget (engine, node))
 		gtk_widget_hide (info->widget);
 }
 
@@ -2051,6 +2076,7 @@ bonobo_ui_engine_sync (BonoboUIEngine   *engine,
 				printf ("-- just syncing state --\n");
 #endif
 				ss (sync, a, cmd_node, b->data, parent);
+
 				(*pos)++;
 			} else {
 				NodeInfo   *info;
@@ -2070,7 +2096,7 @@ bonobo_ui_engine_sync (BonoboUIEngine   *engine,
 					widget, bonobo_ui_node_get_name (a), *pos);
 #endif
 
-				info->widget = widget;
+				info->widget = widget ? gtk_widget_ref (widget) : NULL;
 				if (widget) {
 					bonobo_ui_engine_widget_set_node (
 						sync->engine, widget, a);
@@ -2109,9 +2135,9 @@ bonobo_ui_engine_sync (BonoboUIEngine   *engine,
 					g_warning ("non dirty node, but widget mismatch "
 						   "a: '%s:%s', b: '%s:%s' '%p'",
 						   bonobo_ui_node_get_name (a),
-						   bonobo_ui_node_get_attr (a, "name"),
+						   bonobo_ui_node_peek_attr (a, "name"),
 						   bn ? bonobo_ui_node_get_name (bn) : "NULL",
-						   bn ? bonobo_ui_node_get_attr (bn, "name") : "NULL",
+						   bn ? bonobo_ui_node_peek_attr (bn, "name") : "NULL",
 						   info->widget);
 				}
 			}
@@ -2233,11 +2259,10 @@ seek_dirty (BonoboUIEngine *engine,
  **/
 void
 bonobo_ui_engine_update_node (BonoboUIEngine *engine,
+			      BonoboUISync   *sync,
 			      BonoboUINode   *node)
 {
-	BonoboUISync *sync = find_sync_for_node (engine, node);
-
-	if ((sync = find_sync_for_node (engine, node))) {
+	if (sync) {
 		if (bonobo_ui_sync_is_recursive (sync))
 			seek_dirty (engine, sync, node);
 		else 
@@ -2359,26 +2384,41 @@ bonobo_ui_engine_update (BonoboUIEngine *engine)
 	if (engine->priv->frozen)
 		return;
 
+	access ("Bonobo: UI engine update - start", 0);
+
 	for (l = engine->priv->syncs; l; l = l->next)
 		bonobo_ui_sync_stamp_root (l->data);
 
+	access ("Bonobo: UI engine update - after stamp", 0);
+
 	move_dirt_cmd_to_widget (engine);
 
+	access ("Bonobo: UI engine update - after dirt transfer", 0);
+
 /*	bonobo_ui_engine_dump (priv->win, "before update");*/
 
 	for (node = bonobo_ui_node_children (engine->priv->tree->root);
 	     node; node = bonobo_ui_node_next (node)) {
+		BonoboUISync *sync;
 
 		if (!bonobo_ui_node_get_name (node))
 			continue;
 
-		bonobo_ui_engine_update_node (engine, node);
+		sync = find_sync_for_node (engine, node);
+
+		bonobo_ui_engine_update_node (engine, sync, node);
 	}
 
+	access ("Bonobo: UI engine update - after update nodes", 0);
+
 	update_commands_state (engine);
 
+	access ("Bonobo: UI engine update - after cmd state", 0);
+
 	process_state_updates (engine);
 
+	access ("Bonobo: UI engine update - end", 0);
+
 /*	bonobo_ui_engine_dump (priv->win, "after update");*/
 }
 
@@ -2456,7 +2496,7 @@ bonobo_ui_engine_build_control (BonoboUIEngine *engine,
 
 /*	fprintf (stderr, "Type on '%s' '%s' is %d widget %p\n",
 		 bonobo_ui_node_get_name (node),
-		 bonobo_ui_node_get_attr (node, "name"),
+		 bonobo_ui_node_peek_attr (node, "name"),
 		 info->type, info->widget);*/
 
 	return control;
@@ -2850,7 +2890,7 @@ bonobo_ui_engine_stamp_root (BonoboUIEngine *engine,
 
 	info = bonobo_ui_xml_get_data (engine->priv->tree, node);
 
-	info->widget = widget;
+	info->widget = widget ? gtk_widget_ref (widget) : NULL;
 	info->type |= ROOT_WIDGET;
 
 	bonobo_ui_engine_widget_attach_node (widget, node);
